package classexplorer;

/*
 * IHM du programme
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import classexplorer.panel.*;

public class Fenetre extends JFrame {

    /* Attributs privés */
    private JTextField txtClassName = new JTextField(20);
    private JButton bSearch         = new JButton("Chercher");
    private APIClassesPanel apiClassesPanel = new APIClassesPanel(this);
    private JTabbedPane tabbedPane  = new JTabbedPane();
    private ClassInformationsPanel panClassInformations = new ClassInformationsPanel();
    private ClassConstructorsPanel panClassConstructors = new ClassConstructorsPanel();
    private ClassFieldsPanel panClassFields = new ClassFieldsPanel();
    private ClassMethodsPanel panClassMethods = new ClassMethodsPanel();
    private JarClassListPanel panJarClassList = new JarClassListPanel(this);

    /* Constructeur */
    public Fenetre() {
        initializeTabbedPane();
        buildGUI();
        addListeners();
    }

    /* Méthode privée qui construit l'IHM */
    private void buildGUI() {
        this.setTitle("Explorateur de classes java");
        this.setPreferredSize(new Dimension(800, 600));
        this.pack();
        this.setLocationRelativeTo(null);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Autres composants
        Container container = this.getContentPane();
        JPanel pan = new JPanel();
        pan.add(new JLabel("Nom de la classe :"));
        pan.add(txtClassName);
        pan.add(bSearch);
        container.add(pan, BorderLayout.NORTH);
        
        JSplitPane splitPane = new JSplitPane();
        splitPane.setBorder(new javax.swing.border.EmptyBorder(0, 0, 0, 0));
        splitPane.setLeftComponent(apiClassesPanel);
        splitPane.setRightComponent(tabbedPane);
        container.add(splitPane, BorderLayout.CENTER);
    }

    /* Méthode privée qui ajoute les écouteurs d'évènements */
    private void addListeners() {
        // Les Panels se rafraichissent à chaque entrée
        for (final JPanel panel : new JPanel[] {panClassInformations, panClassConstructors, panClassFields, panClassMethods, panJarClassList}) {
            panel.addFocusListener(new FocusAdapter() {
                @Override
                public void focusGained(FocusEvent e) {
                    panel.validate();
                }
            });
        }

        // bSearch cherche la classe tapée
        bSearch.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent evt) {
                String className = txtClassName.getText().trim();
                if (className.length() == 0) {
                    Toolkit.getDefaultToolkit().beep();
                    return;
                }
                resetTabbedPane();
                initializeTabbedPane();
                try {
                    findInformations(Class.forName(className));
                }
                catch (Exception e) {
                    // La classe n'existe pas
                    JOptionPane.showMessageDialog(null, "Classe introuvable", "ClassExplorer", JOptionPane.ERROR_MESSAGE);
                    return;
                }
            }
        });

        // Taper la touche Enter sur la zone de texte txtClassName produit un click sur bSearch
        txtClassName.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent evt) {
                bSearch.doClick();
            }
        });
    };

    /* Incite les panels à rechercher des informations sur la classe */
    private void findInformations(Class c) throws Exception {
        panClassInformations.findInformations(c);
        panClassConstructors.findInformations(c);
        panClassFields.findInformations(c);
        panClassMethods.findInformations(c);
        apiClassesPanel.selectNode(c.getCanonicalName());
    }

    /* Méthode privée qui initialise l'onglet des informations sur la classe recherchée */
    private void initializeTabbedPane() {
        tabbedPane.add("Informations", new JScrollPane(panClassInformations, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));        
        tabbedPane.add("Constructeurs", panClassConstructors);
        tabbedPane.add("Attributs", panClassFields);
        tabbedPane.add("Méthodes", panClassMethods);
        tabbedPane.add("Jar", panJarClassList);
    }

    /* Méthode privée qui vide tous les onglets */
    private void resetTabbedPane() {
        tabbedPane.removeAll();
        panClassInformations.removeAll();
        panClassConstructors.removeAll();
        panClassMethods.removeAll();
        panClassFields.removeAll();
    }

    // Impose un classe à recherher
    public void searchFor(Class c) throws Exception {
        resetTabbedPane();
        initializeTabbedPane();
        txtClassName.setText(c.getCanonicalName());
        findInformations(c);
    }
}