package classexplorer.panel;

/*
 * Panel pour afficher une arborescence des classes de l'API standard
 */

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import classexplorer.util.APIClasses;
import classexplorer.Fenetre;

public class APIClassesPanel extends JPanel {

    /* Attributs */
    private DefaultMutableTreeNode treeNode = new DefaultMutableTreeNode("Chargement des packages en cours...");
    private DefaultTreeModel treeModel = new DefaultTreeModel(treeNode);
    private JTree tree = new JTree(treeModel);
    private Fenetre caller;

    /* Constructeur */
    public APIClassesPanel(Fenetre caller) {
        this.caller = caller;

        buildGUI();
        createTreePanel();
        addListeners();
    }

    /* Sélectionne un noeud donné dans le JTree */
    public void selectNode(String className) {
        //TreePath path = new TreePath(new String[]{className.substring(0, className.lastIndexOf(".")), className});
        //tree.setSelectionPath(path);
    }

    /* Construit l'IHM */
    private void buildGUI() {
        this.setBorder(new javax.swing.border.TitledBorder("Classes de l'API Standard"));
        this.setLayout(new BorderLayout());
        this.add(new JScrollPane(tree), BorderLayout.CENTER);
    }   

    /* Crée le panel de l'arborescence */
    private void createTreePanel() {
        tree.setRootVisible(true);
        new Thread() {
            @Override
            public void run() {
                for (String packageName : APIClasses.getPackageList("java")) {
                    createTree(packageName, treeNode);
                    treeModel.reload();
                }
                tree.setRootVisible(false);
            }
        }.start();
    }

    /* Crée l'arborescnce des packages */
    private void createTree(String pack, DefaultMutableTreeNode rootNode) {
        DefaultMutableTreeNode packNode = new DefaultMutableTreeNode(pack);
        rootNode.add(packNode);

        // Ajout des classes
        for (String className : APIClasses.getPackageClasses(pack)) {
            packNode.add(new DefaultMutableTreeNode(className));
        }
    }

    /* Ajoute l'écouteur de sélection au JTree */
    private void addListeners() {
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            @Override
            public void valueChanged(TreeSelectionEvent evt) {
                TreePath path = evt.getPath();
                if (path.getPathCount() == 3) {
                    String className = path.toString();
                    className = className.substring(className.lastIndexOf(",")+1, className.length()-1).trim();

                    try {
                        Class c = Class.forName(className);
                        caller.searchFor(c);
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        });
    }
}