package classexplorer.panel;

/*
 * @Class:  JarClassListPanel, fournit des informations sur les méthodes publiques de la classe recherchée
 * @Author: Chatbour
 */

import classexplorer.Fenetre;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import classexplorer.util.JarLoader;
import classexplorer.util.FileSelection;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class JarClassListPanel extends JPanel {

    /* Attributs privés */
    private JarLoader jarLoader;
    private FileSelection jarSelectPanel = new FileSelection("jar", "Jar Archive");
    private DefaultListModel listModel = new DefaultListModel();
    private DefaultListSelectionModel selectionModel = new DefaultListSelectionModel();
    private JList classList = new JList();
    private Fenetre caller;

    /* Constructeur de la classe */
    public JarClassListPanel(Fenetre caller) {
        this.caller = caller;
        buildGUI();
        addListeners();
    }

    /* Consruit l'IHM */
    private void buildGUI() {
        this.setBackground(Color.WHITE);
        this.setLayout(new BorderLayout());
        JPanel pan = new JPanel();
        pan.add(new JLabel("jar :"));
        pan.add(jarSelectPanel);
        this.add(pan, BorderLayout.NORTH);
        classList.setModel(listModel);
        classList.setSelectionModel(selectionModel);
    }

    /* Ajoute les Listeners */
    private void addListeners() {
        // Effectuer une action après la sélection du jar
        jarSelectPanel.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent evt) {
                String jar = jarSelectPanel.getFullPath();
                if (jar == null) {
                    return;
                }
                jarLoader = new JarLoader(jar);
                findInformations();
            }
        });

        // Sélectionner une classe dans la liste produit le chargement de ses informations
        selectionModel.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
        selectionModel.addListSelectionListener(new ListSelectionListener() {
            @Override
            public void valueChanged(ListSelectionEvent evt) {
                if (evt.getValueIsAdjusting()) {
                    String className = (String)(classList.getSelectedValue());
                    try {
                        Class c = jarLoader.loadClass(className);
                        System.out.println(className);
                        caller.searchFor(c);
                    }
                    catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
        });        
    };

    /* Retourne un liste des constructeurs de la classe recherchée */
    public void findInformations() {
        classList.setBorder(new javax.swing.border.TitledBorder("Liste des classes"));

        try {
            for (String str : jarLoader.getClassList()) {
                listModel.addElement(str);
            }

            this.add(new JScrollPane(classList), BorderLayout.CENTER);
            this.revalidate();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /* Supprime la table des classes */
    public void removeAll() {
        removeAll();
        buildGUI();
        revalidate();
    }
}