package classexplorer.reflect;

/*
 * Fournit la liste des constructeurs de la classe
 */

import java.lang.reflect.Constructor;
import java.lang.reflect.Modifier;
import java.util.ArrayList;

public class ClassConstructors {

    /* Attributs privés */
    private Class c;

    /* Constructeur de la classe */
    public ClassConstructors(Class c) {
        this.c = c;
    }

    /* Méthode qui retourne un liste des constructeurs de la classe recherchée */
    public ArrayList<String> getConstructorList() {
        Constructor constructors[] = c.getConstructors();
        ArrayList<String> constructorList = new ArrayList<String>();

        // Les paramètres du constructeur
        for (Constructor constructor : constructors) {
            StringBuilder str = new StringBuilder();
            str.append(Modifier.toString(constructor.getModifiers()) + " " + constructor.getName());
            str.append(" (");
            for (Class parameter : constructor.getParameterTypes()) {
                str.append(parameter.getCanonicalName() + ", ");
            }
            str.deleteCharAt(str.length() - 2);
            str.append(") ");
            // Les exception levées par le constructeur
            if (constructor.getExceptionTypes().length != 0) {
                str.append(" throws ");
                for (Class exception : constructor.getExceptionTypes()) {
                    str.append(exception.getName() + ", ");
                }
                str.deleteCharAt(str.length() - 2);
            }
            // On ajoute une entrée dans l'ArrayList
            constructorList.add(str.toString());
        }

        return constructorList;
    }
}