package classexplorer.reflect;

/*
 * Fournit des informations (package, classe mère, interfaces implémentées) de la classe
 */

import java.lang.reflect.Modifier;

public class ClassInformations {

    /* Attributs privés */
    private Class c;

    /* Constructeur de la classe */
    public ClassInformations(Class c){
        this.c = c;
    }

    /* Retroune le package de la classe */
    public String getPackage() {
        try {
            String pack = c.getPackage().getName();
            return pack;
        }
        catch (NullPointerException e) {
            return null;
        }
    }

    /* Retourne la chaine de déclaration de la classe */
    public String getDeclarationString() {
        String modifiers = Modifier.toString(c.getModifiers()) + " " + c.getCanonicalName();        
        String superClass;
        try {
            superClass = "  extends " + c.getSuperclass().getName();
        }
        catch (Exception e) {
            // il doit s'agir d'une interface et non une classe
            superClass = "";
        }
        Class superInterfaces[] = c.getInterfaces();

        StringBuilder interfaces = new StringBuilder();
        if (superInterfaces != null) {
            interfaces.append("  implements ");
            for (int i=0; i<superInterfaces.length; i++) {
                interfaces.append(superInterfaces[i].getName() + ", ");
            }
            interfaces.deleteCharAt(interfaces.length()-2);

            return modifiers + " " + superClass;
        }

        return modifiers + " " + superClass + " " + interfaces.toString();
    }
}