package classexplorer.reflect;

/*
 * Fournit la liste des méthodes publiques de la classe
 */

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;

public class ClassMethods {

    /* Attributs privés */
    private Class c;

    /* Constructeur de la classe */
    public ClassMethods(Class c) {
        this.c = c;
    }

    /* Méthode qui retourne un liste des constructeurs de la classe recherchée */
    public ArrayList<String> getMethodList() {
        Method methods[] = c.getMethods();
        ArrayList<String> methodList = new ArrayList<String>();

        // Les paramètres des méthodes
        for (Method method : methods) {
            StringBuilder str = new StringBuilder();
            str.append(Modifier.toString(method.getModifiers()) + " " + method.getReturnType().getCanonicalName() +  " " + method.getName());
            str.append(" (");
            for (Class parameter : method.getParameterTypes()) {
                str.append(parameter.getCanonicalName() + ", ");
            }
            str.deleteCharAt(str.length() - 2);
            str.append(") ");
            // Les exception levées par le constructeur
            if (method.getExceptionTypes().length != 0) {
                str.append(" throws ");
                for (Class exception : method.getExceptionTypes()) {
                    str.append(exception.getName() + ", ");
                }
                str.deleteCharAt(str.length() - 2);
            }
            // On ajoute une entrée dans l'ArrayList
            methodList.add(str.toString());
        }

        return methodList;
    }
}