package classexplorer.util;

/*
 * Classe qui liste tous les sous-packages et les classes d'un package donné
 */

import java.io.*;
import java.util.jar.*;
import java.util.*;

public class APIClasses {

    public final static String apiJarName = "rt.jar";

    /* Retourne l'emplacement du fichier apiJar */
    public static String getAPIPath() {
        String paths[] = System.getProperty("sun.boot.class.path").split(System.getProperty("path.separator"));
        String APIPath = "";
        int i=0;
        while (APIPath.equals("")) {
            if (paths[i].endsWith(apiJarName)) {
                APIPath = paths[i];
            }
            else {
                i++;
            }
        }

        return APIPath;
    }

    /* Retourne la liste des packages d'un package donné */
    public static ArrayList<String> getPackageList(String packageName) {
        try {
            ArrayList<String> packageList = new ArrayList<String>();
            JarFile apiJar = new JarFile(getAPIPath());
            Enumeration<JarEntry> entries = apiJar.entries();
            while (entries.hasMoreElements()) {
                String entry = entries.nextElement().getName();
                if (entry.startsWith("java")) {
                    String pack = entry.substring(0, entry.lastIndexOf("/")).replaceAll("/", ".");
                    if (!packageList.contains(pack)) {
                        packageList.add(pack);
                    }
                }
            }

            apiJar.close();

            return packageList;
        }
        catch (IOException e) {
            return null;
        }
    }

    /* Retourne la liste des classes d'un package donné */
    public static ArrayList<String> getPackageClasses(String packageName) {
        try {
            ArrayList<String> classList = new ArrayList<String>();
            classList.addAll(getClassesFromJar(new JarFile(getAPIPath()), packageName));

            // Tri de l'ArrayList
            sort(classList);

            return classList;
        }
        catch (IOException e) {
            return null;
        }
    }

    /* Retourne la liste des classes contenues dans un jar */
    public static ArrayList<String> getClassesFromJar(JarFile jar, String packageName) {
        ArrayList<String> classList = new ArrayList<String>();
        Enumeration<JarEntry> entries = jar.entries();

        while (entries.hasMoreElements()) {
            String entry = entries.nextElement().getName();
            if (entry.matches(packageName + "/[^/]*.class")) {
                classList.add(entry.substring(0, entry.length()-6).replaceAll("/", "."));
            }
        }

        // Tri de l'ArrayList
        sort(classList);

        return classList;
    }

    /* Trie un object ArrayList<String> */
    public static void sort(ArrayList<String> list) {
        Object tab[] = list.toArray();
        Arrays.sort(tab);
        list.clear();
        for (Object str : tab) {
            list.add((String)str);
        }
    }
}