package classexplorer.util;

/*
 *  Composant pour sélectionner un fichier
 */

import javax.swing.*;
import java.awt.event.*;
import javax.swing.filechooser.FileFilter;
import java.io.File;
import java.util.ArrayList;

public class FileSelection extends JPanel implements ActionListener {
    /* Attributs privés */
    private JButton bSelect   = new JButton("Parcourir");
    private JTextField txtFilePath = new JTextField(30);
    private String extension;
    private String description;
    private ArrayList<ActionListener> actions = new ArrayList<ActionListener>();

    /* Constructeur */
    public FileSelection(String ex, String des) {
        extension = ex;
        description = des;

        buildGUI();
        bSelect.addActionListener(this);
    }

    /* Méthode privée pour construire l'interface */
    private void buildGUI() {
        txtFilePath.setEditable(false);

        this.add(txtFilePath);
        this.add(bSelect);
    }

    /* Surcharge de la méthode actionPerformed() */
    @Override
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();

        if((JButton)source == bSelect) {
            txtFilePath.setText(getFileName());
            // Exécuter les autres actions
            for (ActionListener action : actions) {
                action.actionPerformed(null);
            }
        }
    }

    /* Renvoie false si txtCheminFichier est vide */
    public boolean isThereAChosenFile() {
        return txtFilePath.getText().isEmpty() == false;
    }

    /* Renvoie le texte de txtCheminFichier */
    public String getFullPath() {
        if (isThereAChosenFile()) {
            return txtFilePath.getText();
        }
        else {
            return null;
        }
    }

    /* Méthode privée qui invite l'utilisateur à sélectionner un fichier */
    private String getFileName() {
        JFileChooser fileChooser = new JFileChooser(new File("."));
        fileChooser.setFileFilter(new FileFilter() {
            @Override
            public boolean accept(File file) {
                if (file.isDirectory() || file.getName().endsWith(extension)) {
                    return true;
                }
                return false;
            }
            @Override
            public String getDescription() {
                return description;
            }
        });
        fileChooser.showOpenDialog(null);
        if (fileChooser.getSelectedFile() != null) {
            return fileChooser.getSelectedFile().getAbsolutePath();
        }
        else {
            return null;
        }
    }

    /* Méthode qui ajoute un ActionListener au bouton */
    public void addActionListener(ActionListener listener) {
        actions.add(listener);
    }
}